﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

namespace ML4024
{
    public class SFPWrapper
    {
        UInt16 Instance = 0;          // Assuming USB instance is 0
        public bool isConnected = false;
        const string DLLPath = @"DLL\\ML4024.dll";
        #region DLL imports

        [DllImport(DLLPath, EntryPoint = "ConnectToHost")]
        private static extern bool apiConnect(UInt16 Instance);

        [DllImport(DLLPath, EntryPoint = "Disconnect")]
        private static extern bool apiDisconnect(UInt16 Instance);

        [DllImport(DLLPath, EntryPoint = "P3V3_Current_Monitor")]
        private static extern bool apiP3V3CurrentMonitor(UInt16 Instance, ref double data);

        [DllImport(DLLPath, EntryPoint = "MOD_ABS")]
        private static extern bool apiMOD_ABS(UInt16 Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "RX_LOS")]
        private static extern bool apiRXLOS(UInt16 Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "TX_DIS")]
        private static extern bool apiTXDIS(UInt16 Instance, bool asserted);

        [DllImport(DLLPath, EntryPoint = "TX_Fault")]
        private static extern bool apiTXFault(UInt16 Instance, ref bool status);

        [DllImport(DLLPath, EntryPoint = "RS0")]
        private static extern bool apiRS0(UInt16 Instance, bool asserted);

        [DllImport(DLLPath, EntryPoint = "RS1")]
        private static extern bool apiRS1(UInt16 Instance, bool asserted);

        [DllImport(DLLPath, EntryPoint = "I2CWrite")]
        private static extern bool apiI2C_Write(UInt16 Instance, byte SlaveAddress, byte registerAddress, byte RegisterValue);

        [DllImport(DLLPath, EntryPoint = "I2CRead")]
        private static extern bool apiI2C_Read(UInt16 Instance, byte SlaveAddress, byte registerAddress, byte[] inBuff, byte Length);

        [DllImport(DLLPath, EntryPoint = "GetVCC")]
        private static extern bool apiGet_VCC(UInt16 Instance, ref double Data);

        #endregion

        public bool Connect()
        {
            isConnected = apiConnect(Instance);
            return isConnected;
        }

        public bool Disconnect()
        {
            isConnected = !apiDisconnect(Instance);
            return !isConnected;
        }

        public bool CurrentMonitor(ref double data)
        {
            if (isConnected)
                return apiP3V3CurrentMonitor(Instance, ref data);

            return false;
        }

        public bool MOD_ABS(ref bool status)
        {
            if (isConnected)
                apiMOD_ABS(Instance, ref status);

            return false;
        }
        public bool RX_LOS(ref bool status)
        {
            if (isConnected)
                apiRXLOS(Instance, ref status);

            return false;
        }
        public bool TX_DIS(bool asserted)
        {
            if (isConnected)
                return apiTXDIS(Instance, asserted);

            return false;
        }

        public bool I2C_Write(byte SlaveAddress, byte registerAddress, byte RegisterValue)
        {
            if (isConnected)
                return apiI2C_Write(Instance, SlaveAddress, registerAddress, RegisterValue);   // write the register address and the value to be written in that address

            return false;
        }

        public bool I2C_Read(byte SlaveAddress, byte registerAddress, ref byte[] ReadBuff, byte Length)
        {
            if (isConnected)
                return apiI2C_Read(Instance, SlaveAddress, registerAddress, ReadBuff, Length);

            return false;
        }

        public bool Get_VCC(ref double Data)
        {
            if (isConnected)
                return apiGet_VCC(Instance, ref Data);

            return false;
        }
    }
}
